# Generate SWIG files for Python interface of DOLFIN
#
# Copyright (C) 2012-2016 Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

# System imports
import os
import re
import glob
import time
import sys
import io


# Add local site-packages to path,
# NOTE: We need to prepend it so systemwide installed dolfin is not picked up
sys.path.insert(0, os.path.abspath("site-packages"))

from dolfin_utils.cppparser import *

# Local imports
from codesnippets import swig_cmakelists_str, copyright_statement, module_template

# No implicit relative import in python 3
BASE_PACKAGE_NAME = "dolfin.cpp" if sys.version_info[0] == 3 else ""

# Create time info for labeling generated code
_local_time = time.localtime()
_date_form = dict(year = _local_time.tm_year,
                  month = _local_time.tm_mon,
                  day = _local_time.tm_mday)

# Create form for copyright statement to a SWIG interface file
copyright_form_swig = dict(comment = r"//", holder="Johan Hake")

# reg exp pattern
_header_pattern = re.compile("#include +<(dolfin/\w+/\w+\.h)>")
_submodule_pattern = re.compile("#include +<dolfin/(\w+)/\w+\.h>")
_module_pattern = re.compile("// +SWIG +module +(\w+)")

# Get top DOLFIN directory.
script_rel_path = os.sep.join(__file__.split(os.sep)[:-1])
script_rel_path = script_rel_path or "."
dolfin_dir = os.path.abspath(os.path.join(script_rel_path, os.pardir, os.pardir))

global swig_dir
swig_dir = os.path.abspath(os.path.join(dolfin_dir, "dolfin", "swig"))
org_swig_dir = swig_dir


def extract_module_header_files(submodule, excludes):
    """
    Extract header files for a submodule
    """
    # Read dolfin submodule include file
    fn = os.path.join(dolfin_dir, "dolfin", submodule,
                      "dolfin_%s.h" % submodule)
    with open(fn) as f:
        code = f.read()

    # Extract all headers
    all_headers = re.findall(_header_pattern, code)

    # Filter with excludes
    return [header for header in all_headers
            if header.split("/")[-1] not in excludes]


def generate_submodule_info(excludes):
    """
    Check the passed module to submodule mapping and sort it
    Creates the reverse mapping together with additional info
    about each submodule
    """
    # Extract original modules from dolfin.h
    # NOTE: We need these, in particular the order
    original_submodules = []
    module_to_submodules = OrderedDict()
    fn = os.path.join(dolfin_dir, "dolfin", "dolfin.h")
    with open(fn) as f:
        present_module = ""
        for line in f:
            module_match = re.findall(_module_pattern, line)
            if module_match:
                module = module_match[0]
                module_to_submodules[module] = []
                present_module = module
                continue

            submodule_match = re.findall(_submodule_pattern, line)
            if submodule_match:
                submodule = submodule_match[0]
                original_submodules += [submodule]
                if not present_module:
                    raise RuntimeError("Found a submodule in dolfin.h before a "
                                       "SWIG module was declared.")
                module_to_submodules[present_module].append(submodule)

    # Check that the directory structure of the combined modules
    # corresponds to the above dict, if not generate them
    module_dirs = []
    for module_dir in glob.glob(os.path.join(swig_dir, "modules","*")):
        module_dirs.append(module_dir.split(os.path.sep)[-1])

    # Some sanity checks
    for module_dir in module_dirs:
        if module_dir not in module_to_submodules:
            raise RuntimeError("Found a subdirectory: '%s' under the 'modules' "
                               "directory, which is not listed as a combined "
                               "module." % module_dir)

    for combined_module, modules in module_to_submodules.items():
        if combined_module not in module_dirs:
            os.makedirs(os.path.join(swig_dir, "modules", combined_module))
        for module in modules:
            if module not in original_submodules:
                raise RuntimeError("Found a module: '%s' listed in the '%s' "
                                   "combined module, which is not part of the "
                                   "original DOLFIN modules."
                                   % (module, combined_module))

    # Create a map from original modules to the combined
    submodule_info = OrderedDict()
    not_included = []

    for submodule in original_submodules:
        for module, submodules in module_to_submodules.items():
            if submodule in submodules:
                submodule_info[submodule] = dict(
                    module=module,
                    has_pre=os.path.isfile(
                        os.path.join(org_swig_dir, submodule, "pre.i")),
                    has_post=os.path.isfile(
                        os.path.join(org_swig_dir, submodule, "post.i")),
                    headers=extract_module_header_files(submodule, excludes))
                break
        else:
            not_included.append(submodule)

    # Remove not used submodules
    for submodule in not_included:
        original_submodules.remove(submodule)

    # Sort the submodules according to order in original_submodules
    for module, submodules in module_to_submodules.items():
        ordered_submodules = []
        for submodule in original_submodules:
            if submodule in submodules:
                ordered_submodules.append(submodule)
        module_to_submodules[module] = ordered_submodules

    # Return ordered submodule info
    return module_to_submodules, submodule_info


def extract_swig_modules_dependencies(module_to_submodules, submodule_info):
    """
    Extracts the file dependencies of the SWIG modules
    """
    # OrderedDict of all external dependencies for each SWIG module
    module_info = OrderedDict((module, dict(submodules=submodules,
                                            declared_types=[],
                                            used_types=set(),
                                            dependencies={}
                                            ))
                for module, submodules in module_to_submodules.items())

    # dict of where each dolfin type is declared and in what module and
    # submodule it is included in
    dolfin_type_def = OrderedDict()

    # Add UFC Function
    # FIXME: ufc inheritance is not used for now. The ufc module information
    # FIXME: is globally imported in shared_ptr_classes.i
    dolfin_type_def["ufc::function"] = dict(
        module="",
        submodule="",
        header="ufc.h",
        bases=set(), derived=set())

    # dict mapping submodules to included files
    submodule_files = {}

    # Derived classes (Used if a found base class is not registered
    # when first collected)
    derived_classes = {}

    # Iterate over all SWIG modules
    for module, submodules in module_to_submodules.items():

        #print "Parsing headers for SWIG module:", module
        # Iterate over all submodules in each SWIG module
        for submodule in submodules:

            # Iterate over header files and collect info
            for header_file in submodule_info[submodule]["headers"]:
                # Skip version.h (generated from version.h.in via CMake)
                if os.path.split(header_file)[-1] == "version.h":
                    continue

                # Read code
                with io.open(header_file, encoding="utf-8") as f:
                    code = f.read()

                try:
                    # Extract type info
                    used_types, declared_types = parse_and_extract_type_info(code)
                except Exception as e:
                    print()
                    print("###################")
                    print("ERROR while parsing:", header_file)
                    print("###################")
                    print(e)
                    print()
                    continue

                # Store type info
                for dolfin_type, bases in declared_types.items():  # XXX

                    # Store type information
                    dolfin_type_def[dolfin_type] = dict(module=module,
                                                        submodule=submodule,
                                                        header=header_file,
                                                        bases=bases, derived=set())

                    # Register the class
                    module_info[module]["declared_types"].append(dolfin_type)

                    # Store derived classes
                    for base in bases:
                        # Check if base class has been regisred
                        # (should be as the types are traversed in order of
                        #  declaration )
                        dolfin_type_info = dolfin_type_def.get(base)
                        if dolfin_type_info is None:
                            if base not in derived_classes:
                                derived_classes[base] = set()
                            derived_classes[base].add(dolfin_type)

                        else:
                            # Add derived class
                            dolfin_type_info["derived"].add(dolfin_type)

                # Add collected types to module
                module_info[module]["used_types"].update(
                    used_types)

    # If for one reason or the other a base class was detected which
    # was not registered we add derived information here
    for dolfin_type_name, derived in derived_classes.items():
        dolfin_type = dolfin_type_def.get(dolfin_type_name)
        if dolfin_type is not None:
            dolfin_type["derived"].update(derived)

    # Help functions to recursevily add base and derived information
    # to types
    def add_bases(bases, commulative_bases):
        for base in bases:
            # Check if bas is a dolfin type
            if base in dolfin_type_def:
                commulative_bases.add(base)
                add_bases(dolfin_type_def[base]["bases"], commulative_bases)

    def add_derived(derived_set, commulative_derived):
        for derived in derived_set:
            # All derived should be a dolfin type...
            if derived in dolfin_type_def:
                commulative_derived.add(derived)
                add_derived(dolfin_type_def[derived]["derived"], commulative_derived)

    # Build class hierarchy (We need to import all base and derived classes)
    # First extract all bases to a type
    for dolfin_type in dolfin_type_def:

        # Recursively add base and derived classes
        if dolfin_type_def[dolfin_type]["bases"]:
            new_bases = set()
            add_bases(dolfin_type_def[dolfin_type]["bases"], new_bases)
            dolfin_type_def[dolfin_type]["bases"] = sorted(new_bases)

        if dolfin_type_def[dolfin_type]["derived"]:
            new_derived = set()
            add_derived(dolfin_type_def[dolfin_type]["derived"], new_derived)
            dolfin_type_def[dolfin_type]["derived"] = sorted(new_derived)

    # Collect used dolfin types in each module
    used_dolfin_types = dict((module, set()) for module in module_info)

    # Filter out dolfin types and add derived and bases for each type
    for dolfin_type in dolfin_type_def:

        # Turn all set data into lists
        if isinstance(dolfin_type_def[dolfin_type]["bases"], set):
            dolfin_type_def[dolfin_type]["bases"] = \
                        sorted(dolfin_type_def[dolfin_type]["bases"])
        if isinstance(dolfin_type_def[dolfin_type]["derived"], set):
            dolfin_type_def[dolfin_type]["derived"] = \
                        sorted(dolfin_type_def[dolfin_type]["derived"])

        for module in module_info:
            for used_type in module_info[module]["used_types"]:
                if dolfin_type in used_type:
                    used_dolfin_types[module].add(dolfin_type)

                    # Add bases and derived types
                    used_dolfin_types[module].update(
                        dolfin_type_def[dolfin_type]["bases"])

                    break

    # Over write old used type
    for module in module_info:
        # Update dependencies
        module_info[module]["used_types"] = used_dolfin_types[module]

    # Check external module dependencies
    for present_module in module_info:
        dependencies = module_info[present_module]["dependencies"]
        for dependent_module in module_info:

            # If same module no external dependencies
            if present_module == dependent_module:
                continue

            # Iterate over all dolfin types in dependent modules and check it they
            # are present in the present module
            for dolfin_type in module_info[dependent_module]["declared_types"]:
                # Check for dependency
                if dolfin_type in module_info[present_module]["used_types"]:

                    # Register the dependency
                    submodule = dolfin_type_def[dolfin_type]["submodule"]
                    if submodule not in dependencies:
                        dependencies[submodule] = set()
                    dependencies[submodule].add(
                        dolfin_type_def[dolfin_type]["header"])

        # Need special treatment for template definitions in function/pre.i
        if "function" in dependencies:
            for dolfin_type in ["FunctionSpace", "Function"]:
                dependencies["function"].add(
                    dolfin_type_def[dolfin_type]["header"])

        # Need special treatment to include constants.h if not already included
        if present_module != "common":
            # If no dependencies of common add it
            if "common" not in dependencies:
                dependencies["common"] = set()
            dependencies["common"].add(
                "dolfin/common/constants.h")

        # Over write old submodules dependencies with sorted version
        module_info[present_module]["dependencies"] = \
            sort_submodule_dependencies(dependencies, submodule_info)

    # Return data structures
    return module_info, dolfin_type_def


def write_module_interface_file(module, dependencies, submodule_info, parsed_modules):
    """
    Write the main interface file for SWIG module
    """

    # Generate a form for code template
    module_form = dict(
        module=module,
        MODULE=module.upper(),
        )

    # Create import and include lines for each dependent file
    import_lines, headers_includes, file_dependencies = \
        build_swig_import_info(dependencies, submodule_info,
        package_name=BASE_PACKAGE_NAME,
        parsed_modules=parsed_modules)

    # Add global SWIG interface files
    file_dependencies.append(os.path.join(swig_dir, "modules", module, "module.i"))
    file_dependencies.extend(glob.glob("dolfin/swig/*.i"))
    file_dependencies.extend(glob.glob("dolfin/swig/typemaps/*.i"))

    include_lines = []
    docstring_lines = []

    headers_includes.append("")
    headers_includes.append("// Include types from present module %s" % module)

    for submodule, submod_info in submodule_info.items():

        if submod_info["module"] != module:
            continue

        headers_includes.append("")
        headers_includes.append("// #include types from %s submodule" % submodule)

        include_lines.append("")
        include_lines.append("// %%include types from submodule %s" % submodule)

        # Add docstrings
        docstring_lines.append("%%include \"dolfin/swig/%s/docstrings.i\"" % submodule)

        # Check for pre includes
        if submod_info["has_pre"]:
            include_lines.append(
                "%%include \"dolfin/swig/%s/pre.i\"" % submodule)
            file_dependencies.append(
                os.path.join(swig_dir, submodule, "pre.i"))

        # Add headers
        headers_includes.extend("#include \"%s\"" % header
                                for header in submod_info["headers"])
        include_lines.extend("%%include \"%s\"" % header
                             for header in submod_info["headers"])

        # Check for post includes
        if submod_info["has_post"]:
            include_lines.append(
                "%%include \"dolfin/swig/%s/post.i\"" % submodule)
            file_dependencies.append(
                os.path.join(swig_dir, submodule, "post.i"))

    # Add imports and includes to form
    module_form["imports"] =    "\n".join(import_lines)
    module_form["includes"] =   "\n".join(include_lines)
    module_form["docstrings"] = "\n".join(docstring_lines)
    module_form["headers"] =    "\n".join(headers_includes)

    # Write the generated code
    fn = os.path.join(swig_dir, "modules", module, "module.i")
    with open(fn, "w") as module_file:
        module_file.write(copyright_statement % copyright_form_swig)
        module_file.write(module_template % module_form)

    # Make file dependency paths absolute
    file_dependencies = [os.path.abspath(f) for f in file_dependencies]

    # Write swig interface file dependencies
    fn = os.path.join(swig_dir, "modules", module, "dependencies.txt")
    with open(fn, "w") as dependency_file:
        dependency_file.write(";".join(sorted(file_dependencies)))


def write_swig_cmakelist_file(module):
    """
    Generate the CMakeList.txt file for each module
    """
    swig_cmakelists = """# This file is automatically generated by running
#
#     python cmake/scripts/generate-swig-interface.py
#
""" + swig_cmakelists_str

    # Write swig interface file dependencies
    fn = os.path.join(swig_dir, "modules", module, "CMakeLists.txt")
    with open(fn, "w") as cmakelist_file:
        cmakelist_file.write(swig_cmakelists)


def repr_ordered_dict(data, indent=0):
    """
    Make a readable repr version of an OrderedDict
    """
    sp = "  "*indent
    sep = ",\n" + sp
    if isinstance(data, OrderedDict):
        return "OrderedDict([\n%s%s\n])" % (sp, sep.join(
            "('%s', %s)" % (key, repr_ordered_dict(value, indent+1))
            for key, value in data.items()))
    elif isinstance(data, dict):
        return "dict([\n%s%s\n])" % (sp, sep.join(
            "('%s', %s)" % (key, repr_ordered_dict(value, indent+1))
            for key, value in sorted(data.items())))
    else:
        return repr(data)


def generate_runtime_config_file(dolfin_type_def, module_to_submodules,
                                 submodule_info):
    # Extract all shared_ptr stored classes and store them in a python module
    # and place that under dolfin.compilemodeuls.sharedptrclasses.py

    with open(os.path.join(org_swig_dir, "shared_ptr_classes.i")) as f:
        classes = f.read()
    shared_ptr_classes = re.findall("%shared_ptr\(dolfin::(.+)\)", classes)

    #shared_ptr_classes = filter(lambda x: "NAME" not in x, shared_ptr_classes)
    runtime_file = '''"""
This module contains compiletime information about the
dolfin python library, which can be utilized at runtime.

The file is automatically generated by the generateswigcode.py script
in the dolfin/swig directory."""

try:
    from collections import OrderedDict
except ImportError:
    from dolfin_utils.ordereddict import OrderedDict

# A list of shared_ptr declared classes in dolfin
shared_ptr_classes = %s

# An OrderedDict of all dolfin declared and its meta info
dolfin_type_def = %s

# A map between modules and its submodules
module_to_submodules = %s

# A reverse map between submodules and modules
submodule_info = %s
''' % (sorted(shared_ptr_classes), repr_ordered_dict(dolfin_type_def),
       repr_ordered_dict(module_to_submodules), repr_ordered_dict(submodule_info))

    # FIXME: Create this in build directory
    fn = os.path.join("site-packages", "dolfin", "compilemodules", "swigimportinfo.py")
    with open(fn, "w") as f:
        f.write(runtime_file)


def regenerate_swig_interface(excludes, top_destdir):
    """
    Regenerate the whole swig interface
    """

    global swig_dir

    top_destdir = top_destdir or dolfin_dir
    abs_destdir = top_destdir if os.path.isabs(top_destdir) else os.path.join(dolfin_dir, top_destdir)

    # Check what part of path dolfin_dir has in common abs_destdir:
    dolfin_dirs = dolfin_dir.split(os.sep)
    abs_destdirs = abs_destdir.split(os.sep)
    while dolfin_dirs and abs_destdirs:
        # Check if dolfin and abs_dest dir has the same top directory
        if dolfin_dirs[0] == abs_destdirs[0]:
            dolfin_dirs.pop(0)
            abs_destdirs.pop(0)
        else:
            break

    if not os.path.isdir(abs_destdir):
        raise RuntimeError("%s is not a directory." % abs_destdir)

    # Change swig_dir for generated files
    if top_destdir != dolfin_dir:
        swig_dir = swig_dir.replace(dolfin_dir, top_destdir)

    # Check the submodule order and create a submodule to module mapping
    module_to_submodules, submodule_info = generate_submodule_info(excludes)

    # Create dolfin type info and depdency structures to be used when
    # generating swig interface files
    module_info, dolfin_type_def = extract_swig_modules_dependencies(
        module_to_submodules, submodule_info)

    # Iterate over the modules and create a swig interface file
    submodule_ordered = [submodule for submodules in module_to_submodules.values()
                         for submodule in submodules]
    parsed_modules = []
    for module in module_to_submodules:
        # Sort submodules
        dependencies = sorted(module_info[module]["dependencies"].keys(),
            key=lambda x: submodule_ordered.index(x))
        write_module_interface_file(module, module_info[module]["dependencies"],
                                    submodule_info, parsed_modules)
        write_swig_cmakelist_file(module)
        parsed_modules.append(module)

    # Create a python module with generated type info to be used runtime
    generate_runtime_config_file(dolfin_type_def, module_to_submodules,
                                 submodule_info)


if __name__ == "__main__":

    if len(sys.argv) not in [1,2]:
        raise RuntimeError("expected 0 or 1 argument")

    dest_dir = sys.argv[-1] if len(sys.argv) == 2 else ""

    # User defined list of headers to exclude (add more here)
    excludes = ["LogStream.h"]

    # Regnerate SWIG interface
    regenerate_swig_interface(excludes, dest_dir)



# -*- coding: utf-8 -*-
"""This module defines a jit function that wraps the jit function for
the chosen form compiler."""

# Copyright (C) 2008-2016 Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Anders Logg, 2008-2009.
# Modified by Garth N. Wells, 2011.
# Modified by Martin Sandve Alnæs, 2014-2016.

from __future__ import print_function

__all__ = ["jit"]

from functools import wraps
import sys
import traceback
import six

import ufl
import ffc

# Import SWIG-generated extension module (DOLFIN C++)
import dolfin.cpp as cpp
from dolfin.cpp import MPI, info, parameters, warning, info
from dolfin.common.globalparameters import ffc_default_parameters


def mpi_jit_decorator(local_jit, *args, **kwargs):
    """A decorator for jit compilation

    Use this function as a decorator to any jit compiler function.  In
    a parallel run, this function will first call the jit compilation
    function on the first process. When this is done, and the module
    is in the cache, it will call the jit compiler on the remaining
    processes, which will then use the cached module.

    *Example*
        .. code-block:: python

            def jit_something(something):
                ....

    """
    @wraps(local_jit)
    def mpi_jit(*args, **kwargs):

        # Create MPI_COMM_WORLD wrapper
        mpi_comm = kwargs.get("mpi_comm")
        if mpi_comm is None:
            mpi_comm = cpp.mpi_comm_world()

        # Just call JIT compiler when running in serial
        if MPI.size(mpi_comm) == 1:
            return local_jit(*args, **kwargs)

        # Default status (0 == ok, 1 == fail)
        status = 0

        # Compile first on process 0
        root = MPI.rank(mpi_comm) == 0
        if root:
            try:
                output = local_jit(*args, **kwargs)
            except Exception as e:
                status = 1
                error_msg = str(e)

        # TODO: This would have lower overhead if using the dijitso.jit
        # features to inject a waiting callback instead of waiting out here.
        # That approach allows all processes to first look in the cache,
        # introducing a barrier only on cache miss.
        # There's also a sketch in dijitso of how to make only one
        # process per physical cache directory do the compilation.

        # Wait for the compiling process to finish and get status
        # TODO: Would be better to broadcast the status from root but this works.
        global_status = MPI.max(mpi_comm, status)

        if global_status == 0:
            # Success, call jit on all other processes
            # (this should just read the cache)
            if not root:
                output = local_jit(*args,**kwargs)
        else:
            # Fail simultaneously on all processes,
            # to allow catching the error without deadlock
            if not root:
                error_msg = "Compilation failed on root node."
            cpp.dolfin_error("jit.py",
                             "perform just-in-time compilation of form",
                             error_msg)
        return output

    # Return the decorated jit function
    return mpi_jit


@mpi_jit_decorator
def jit(ufl_object, form_compiler_parameters=None, mpi_comm=None):
    """Just-in-time compile any provided UFL Form or FiniteElement using FFC.

    Default parameters from FFC are overridden by parameters["form_compiler"]
    first and then the form_compiler_parameters argument to this function.
    """
    # Check that form is not empty (workaround for bug in UFL where
    # bilinear form 0*u*v*dx becomes the functional 0*dx)
    if isinstance(ufl_object, ufl.Form) and ufl_object.empty():
        cpp.dolfin_error("jit.py",
                         "perform just-in-time compilation of form",
                         "Form is empty. Cannot pass to JIT compiler")

    # Check DOLFIN build time UFC matches currently loaded UFC
    if cpp.__ufcsignature__ != ffc.ufc_signature():
        cpp.dolfin_error("jit.py",
                         "perform just-in-time compilation of form",
                         "DOLFIN was not compiled against matching"
                         "UFC from current FFC installation.")

    # Prepare form compiler parameters with overrides from dolfin and kwargs
    p = ffc_default_parameters()
    p.update(parameters["form_compiler"])
    p.update(form_compiler_parameters or {})

    # Execute!
    try:
        result = ffc.jit(ufl_object, parameters=p)
    except Exception as e:
        tb_text = ''.join(traceback.format_exception(*sys.exc_info()))
        cpp.dolfin_error("jit.py",
                         "perform just-in-time compilation of form",
                         "ffc.jit failed with message:\n%s" % (tb_text,))

    # Unpack result (ffc.jit returns different tuples based on input type)
    if isinstance(ufl_object, ufl.Form):
        compiled_form, module, prefix = result
        compiled_form = cpp.make_ufc_form(compiled_form)
        return compiled_form, module, prefix
    elif isinstance(ufl_object, ufl.FiniteElementBase):
        ufc_element, ufc_dofmap = result
        ufc_element = cpp.make_ufc_finite_element(ufc_element)
        ufc_dofmap = cpp.make_ufc_dofmap(ufc_dofmap)
        return ufc_element, ufc_dofmap
    elif isinstance(ufl_object, ufl.Mesh):
        ufc_coordinate_mapping = result
        ufc_coordinate_mapping = cpp.make_ufc_coordinate_mapping(ufc_coordinate_mapping)
        return ufc_coordinate_mapping



# -*- coding: utf-8 -*-
"""This module handles the MultiMeshFunction class in Python.
"""
# Copyright (C) 2016 Jørgen Schartum Dokken
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#

__all__ = ["MultiMeshFunction"]

# Import UFL and SWIG-generated extension module (DOLFIN C++)
import ufl
import dolfin.cpp as cpp
import numpy

from dolfin.functions.functionspace import MultiMeshFunctionSpace
from dolfin.functions.function import Function

class MultiMeshFunction(ufl.Coefficient, cpp.MultiMeshFunction):
    """This class represents a multimeshfunction
    :math:`u_h=(u_{h,1}\cross \dots u_{h,N}` in a finite
    element multimeshfunction space
    :math:`V_h=V_{h,1}\cross \dots V_{h,N}`, given by

    .. math::

        u_{h,j}=  \sum_{i=1}^n U_i \phi_{i,j},

    where :math:`\{\phi_{i,j}\}_{i=1}^n` is a basis for :math:`V_{h,j}`,
    and :math:`U` is a vector of expansion coefficients for
    :math:`u_h`.

    *Arguments*
        There is a maximum of two arguments. The first argument must be a
        :py:class:`MultiMeshFunctionSpace
	 <dolfin.cpp.function.MultiMeshFunctionSpace>`.

	The second argument must be a GenericVector and is intended for library
	use only.

    *Examples*
        Create a MultiMeshFunction:

        - from a :py:class:`MultiMeshFunctionSpace
          <dolfin.cpp.function.MultiMeshFunctionSpace>` ``V``

          .. code-block:: python

              f = MultiMeshFunction(V)


        - from a :py:class:`MultiMeshFunctionSpace
          <dolfin.cpp.function.MultiMeshFunctionSpace>` ``V`` and a
          :py:class:`GenericVector <dolfin.cpp.GenericVector>` ``v``

          *Warning: this constructor is intended for internal libray use only.*

          .. code-block:: python

              g = MultiMeshFunction(V, v)

    """

    def __init__(self, *args, **kwargs):
        """Initialize MultiMeshFunction."""
        # Initial quick check for valid arguments (other checks
        # sprinkled below)
        if len(args) == 0:
            raise TypeError("expected 1 or more arguments")
        # Type switch on argument types
        if isinstance(args[0], MultiMeshFunction):
            other = args[0]
            if len(args) == 1:
                # Copy constructor used to be here
                raise RuntimeError("Use 'MultiMeshFunction.copy(deepcopy=True)' for copying.")
            else:
                raise NotImplementedError
        elif isinstance(args[0], cpp.MultiMeshFunction):
            raise NotImplementedError
        elif isinstance(args[0], cpp.MultiMeshFunctionSpace):
            V = args[0]
            # If initialising from a FunctionSpace
            if len(args) == 1:
                # If passing only the FunctionSpace
                self.__init_from_multimeshfunction_space(V)
            elif len(args) == 2:
                other = args[1]
                if isinstance(other, cpp.MultiMeshFunction):
                    raise NotImplementedError
                else:
                    self.__init_from_function_space_and_function(V, other)
            else:
                raise TypeError("too many arguments")

            # Keep a reference of the functionspace with additional attributes
            self._V = V
        else:
            raise TypeError("expected a MultiMeshFunctionSpace or a MultiMeshFunction as argument 1")


    def __init_from_multimeshfunction_space(self, V):
        cpp.MultiMeshFunction.__init__(self, V)
        ufl.Coefficient.__init__(self, V._parts[0].ufl_function_space(),
                                 count=self.id())

    def __init_from_function_space_and_function(self, V, other):
        cpp.MultiMeshFunction.__init__(self, V, other)
        ufl.Coefficient.__init__(self, V._parts[0].ufl_function_space(),
                                 count=self.id())

    def function_space(self):
        return self._V

    def assign(self, rhs):
        """
        Parameters: 
            rhs: A dolfin.MultiMeshFunction
        """
        # Assign a MultiMeshFunction into a MultiMeshFunction
        if isinstance(rhs, MultiMeshFunction):
            self.vector()[:]= rhs.vector()[:]
        else:
            raise TypeError("expected a MultiMeshFunction as argument.")

    def part(self, i, deepcopy=False):
        f = Function(super(MultiMeshFunction, self).part(i, deepcopy))
        f.rename(super(MultiMeshFunction, self).name(), super(MultiMeshFunction, self).label())
        return f

    def parts(self, deepcopy=False):
        """
        Generator for MultiMeshFunction
        """
        for part in range(self._V.num_parts()):
            yield self.part(part, deepcopy)

    def interpolate(self, v):
        """
        Interpolate function.

        *Arguments*
            v
              a :py:class:`MultiMeshFunction <dolfin.functions.function.MultimeshFunction>` or
              an :py:class:`Expression <dolfin.functions.expression.Expression'>
        *Example of usage*
            .. code-block:: python

                V = MultiMeshFunctionSpace(multimesh, "Lagrange", 1)
                v = MultiMeshFunction(V)
                w = Expression("sin(pi*x[0])")
                v.interpolate(w)
        """
        # Developer note: This version involves a lot of copying
        # and should be changed at some point.
        # Developer note: Interpolate does not set inactive dofs to zero,
        # and should be fixed
        # Check argument
        if isinstance(v, cpp.GenericFunction):
            for i, vp in enumerate(self.parts(deepcopy=True)):
                vp.interpolate(v)
                self.assign_part(i, vp)

        elif isinstance(v, MultiMeshFunction):
            # Same multimesh required for interpolation
            # Developer note: Is this test necessary?
            if  self._V.multimesh().id() != v._V.multimesh().id():
                raise RuntimeError("MultiMeshFunctions must live on same MultiMesh")
            for i, vp in enumerate(self.parts(deepcopy=True)):
                vmm = v.part(i, deepcopy=True)
                vp.interpolate(vmm)
                self.assign_part(i, vp)
        else:
            raise TypeError("Expected an Expression or a MultiMeshFunction.")

        # Set inactive dofs to zero
        for part in range(self._V.num_parts()):
            dofs = self._V.dofmap().inactive_dofs(self._V.multimesh(),part)
            self.vector()[dofs]=0



# -*- coding: utf-8 -*-
# Copyright (C) 2010-2012 Marie E. Rognes
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Johan Hake 2011
# Modified by Anders Logg 2011
# Modified by Jan Blechta 2015

import ufl
import ufl.algorithms.elementtransformations

import dolfin.cpp as cpp
from dolfin import FunctionSpace, Function, Argument, MultiMeshFunction


__all__ = ["derivative", "adjoint", "increase_order", "tear"]

def adjoint(form, reordered_arguments=None):

    # Call UFL directly if new arguments are provided directly
    if reordered_arguments is not None:
        return ufl.adjoint(form, reordered_arguments=reordered_arguments)

    # Extract form arguments
    arguments = form.arguments()
    if any(arg.part() != None for arg in arguments):
        cpp.dolfin_error("formmanipulation.py",
                         "compute adjoint of form",
                         "parts not supported")
    if not (len(arguments) == 2):
        cpp.dolfin_error("formmanipulation.py",
                         "compute adjoint of form",
                         "Form is not bilinear")

    # Define new Argument(s) in the same spaces
    # (NB: Order does not matter anymore here because number is absolute)
    v_1 = Argument(arguments[1].function_space(), arguments[0].number(), arguments[0].part())
    v_0 = Argument(arguments[0].function_space(), arguments[1].number(), arguments[1].part())
    # Also copy the extended part of the argument for MultiMesh-functionality
    if hasattr(arguments[0], '_V_multi'):
        v_1._V_multi = arguments[1]._V_multi
        v_0._V_multi = arguments[0]._V_multi

    # Call ufl.adjoint with swapped arguments as new arguments
    return ufl.adjoint(form, reordered_arguments=(v_1, v_0))

adjoint.__doc__ = ufl.adjoint.__doc__

def derivative(form, u, du=None, coefficient_derivatives=None):
    if du is None:
        # Get existing arguments from form and position the new one with the next argument number
        form_arguments = form.arguments()

        number = max([-1] + [arg.number() for arg in form_arguments]) + 1

        if any(arg.part() is not None for arg in form_arguments):
            cpp.dolfin_error("formmanipulation.py",
                             "compute derivative of form",
                             "Cannot automatically create new Argument using "
                             "parts, please supply one")
        part = None

        if isinstance(u, (Function, MultiMeshFunction)):
            V = u.function_space()
            du = Argument(V, number, part)
        elif isinstance(u, (list,tuple)) and all(isinstance(w, Function) for w in u):
            cpp.dolfin_error("formmanipulation.py",
                             "take derivative of form w.r.t. a tuple of Coefficients",
                             "Take derivative w.r.t. a single Coefficient on "\
                             "a mixed space instead.")
        else:
            cpp.dolfin_error("formmanipulation.py",
                             "compute derivative of form w.r.t. '%s'" % u,
                             "Supply Function as a Coefficient")

    return ufl.derivative(form, u, du, coefficient_derivatives)

derivative.__doc__ = ufl.derivative.__doc__
derivative.__doc__ += """

    A tuple of Coefficients in place of a single Coefficient is not
    supported in DOLFIN. Supply rather a Function on a mixed space in
    place of a Coefficient.
    """

def increase_order(V):
    """
    For a given function space, return the same space, but with a
    higher polynomial degree
    """
    mesh = V.mesh()
    element = ufl.algorithms.elementtransformations.increase_order(V.ufl_element())
    constrained_domain = V.dofmap().constrained_domain
    return FunctionSpace(mesh, element, constrained_domain=constrained_domain)

def change_regularity(V, family):
    """
    For a given function space, return the corresponding space with
    the finite elements specified by 'family'. Possible families
    are the families supported by the form compiler
    """
    mesh = V.mesh()
    element = ufl.algorithms.elementtransformations.change_regularity(V.ufl_element(), family)
    constrained_domain = V.dofmap().constrained_domain
    return FunctionSpace(mesh, element, constrained_domain=constrained_domain)

def tear(V):
    """
    For a given function space, return the corresponding discontinuous
    space
    """
    return change_regularity(V, "DG")



# -*- coding: utf-8 -*-
# Copyright (C) 2017 Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import shutil

def process():
    """Copy demo rst files (C++ and Python) from the DOLFIN source tree
    into the demo source tree, and process file with pylit

    """

    # Create demo CMakeLists.txt files
    current_dir = os.getcwd()
    os.chdir("../../")
    os.system(sys.executable + ' ./cmake/scripts/generate-cmakefiles.py')
    os.chdir(current_dir)

    # Check that we can find pylint.py for converting foo.py.rst to
    # foo.py
    pylit_parser = "../../utils/pylit/pylit.py"
    if os.path.isfile(pylit_parser):
        pass
    else:
        raise RuntimeError("Cannot find pylit.py")

    # Directories to scan
    subdirs = ["../../demo/documented"]

    # Iterate over subdirectories containing demos
    for subdir in subdirs:

        # Get list of demos (demo name , subdirectory)
        demos = [(dI, os.path.join(subdir, dI)) for dI in os.listdir(subdir) if os.path.isdir(os.path.join(subdir, dI))]

        # Iterate over demos
        for demo, path in demos:

            # Build list of rst files in demo directory
            rst_files_common = [f for f in os.listdir(path) if os.path.splitext(f)[1] == ".rst" ]

            # Make demo doc directory
            demo_dir = os.path.join('./demos/', demo)
            if not os.path.exists(demo_dir):
                os.makedirs(demo_dir)

            for f in rst_files_common:
                shutil.copy(os.path.join(path, f), demo_dir)

            # Process C++ and Python versions
            for version in ("cpp", "python"):

                # Get path to demo source directory (cpp/python) and
                # check that it exists
                version_path = os.path.join(path, version)
                if not os.path.isdir(version_path):
                    continue

                # Build list of rst files in demo source directory
                rst_files = [f for f in os.listdir(version_path) if os.path.splitext(f)[1] == ".rst" ]

                # Create directory in documentation tree for demo
                demo_dir = os.path.join('./demos/', demo, version)
                if not os.path.exists(demo_dir):
                    os.makedirs(demo_dir)

                # Copy rst files into documentation demo directory and
                # process with Pylit
                for f in rst_files:
                    shutil.copy(os.path.join(version_path, f), demo_dir)

                    # Run pylit on cpp.rst and py.rst files (file with
                    # 'double extensions')
                    if os.path.splitext(os.path.splitext(f)[0])[1] in (".py", ".cpp", ".ufl"):
                        rst_file = os.path.join(demo_dir, f)
                        command = pylit_parser + " " + rst_file
                        ret = os.system(command)
                        if not ret == 0:
                            raise RuntimeError("Unable to convert rst file to a .cpp/py ({})".format(f))

                # Copy CMakeLists.txt file (if it exists)
                cmake_file = os.path.join(version_path, "CMakeLists.txt")
                if os.path.isfile(cmake_file):
                    shutil.copy(cmake_file, demo_dir)



# -*- coding: utf-8 -*-
# Copyright (C) 2010 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2010-02-14
# Last changed: 2010-02-14

import os

__all__ = ["memory_usage"]

def memory_usage(as_string=True):
    """
    Return memory usage of current process. Note that this function
    only works on Linux systems. If the flag as_string is true, memory
    usage is returned as a string. Otherwise, a tuple of integers is
    returned with current memory usage and maximum memory usage in kB.
    """

    # Try reading /proc
    try:
        pid = os.getpid()
        status = open("/proc/%d/status" % pid).read()
    except:
        return "Failed to get memory usage, chances are you are not running GNU/Linux."

    # Parse values
    vmsize = int(status.split("VmSize:")[1].split("kB")[0])
    vmpeak = int(status.split("VmPeak:")[1].split("kB")[0])

    # Return values
    if as_string:
        return "Memory usage is %d kB (peak usage is %d kB)" % (vmsize, vmpeak)
    else:
        return (vmsize, vmpeak)



# -*- coding: utf-8 -*-
"""The function module of dolfin"""

from dolfin.functions import multimeshfunction
from dolfin.functions import functionspace
from dolfin.functions import function
from dolfin.functions import constant
from dolfin.functions import expression
from dolfin.functions import specialfunctions

from .multimeshfunction import *
from .functionspace import *
from .function import *
from .constant import *
from .expression import *
from .specialfunctions  import *

# NOTE: The automatic documentation system in DOLFIN requires to _not_ define
# NOTE: classes or functions within this file. Use separate modules for that
# NOTE: purpose.

__all__ = functionspace.__all__ + function.__all__ + constant.__all__ + \
          expression.__all__ + specialfunctions.__all__ + \
          multimeshfunction.__all__



# -*- coding: utf-8 -*-
"""This module handles the Function class in Python.
"""
# Copyright (C) 2009-2014 Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Martin Sandve Alnæs 2013-2014
# Modified by Anders Logg 2015

__all__ = ["Function", "TestFunction", "TrialFunction", "Argument",
           "TestFunctions", "TrialFunctions"]

from six import string_types
import types

# Import UFL and SWIG-generated extension module (DOLFIN C++)
import ufl
from ufl import product
from ufl.utils.indexflattening import flatten_multiindex, shape_to_strides
import dolfin.cpp as cpp
import numpy

from dolfin.functions.functionspace import FunctionSpace
from dolfin.functions.constant import Constant
from six.moves import xrange as range

def _assign_error():
    cpp.dolfin_error("function.py",
                     "assign function",
                     "Expects only linear combinations of Functions in "\
                     "the same FunctionSpaces")

def _check_mul_and_division(e, linear_comb, scalar_weight=1.0, multi_index=None):
    """
    Utility func for checking division and multiplication of a Function
    with scalars in linear combinations of Functions
    """
    from ufl.constantvalue import ScalarValue
    from ufl.classes import ComponentTensor, MultiIndex, Indexed
    from ufl.algebra import Division, Product, Sum
    #ops = e.ufl_operands

    # FIXME: What should be checked!?
    # martinal: This code has never done anything sensible,
    #   but I don't know what it was supposed to do so I can't fix it.
    #same_multi_index = lambda x, y: (x.ufl_free_indices == y.ufl_free_indices \
    #                        and x.ufl_index_dimensions == y.ufl_index_dimensions)

    assert isinstance(scalar_weight, float)

    # Split passed expression into scalar and expr
    if isinstance(e, Product):
        for i, op in enumerate(e.ufl_operands):
            if isinstance(op, ScalarValue) or \
                   (isinstance(op, Constant) and op.value_size()==1):
                scalar = op
                expr = e.ufl_operands[1-i]
                break
        else:
            _assign_error()

        scalar_weight *= float(scalar)
    elif isinstance(e, Division):
        expr, scalar = e.ufl_operands
        if not (isinstance(scalar, ScalarValue) or \
                isinstance(scalar, Constant) and scalar.value_rank()==1):
            _assign_error()
        scalar_weight /= float(scalar)
    else:
        _assign_error()

    # If a CoefficientTensor is passed we expect the expr to be either
    # a Function or another ComponentTensor, where the latter wil
    # result in a recursive call
    if multi_index is not None:
        assert isinstance(multi_index, MultiIndex)
        assert isinstance(expr, Indexed)

        # Unpack Indexed and check equality with passed multi_index
        expr, multi_index2 = expr.ufl_operands
        assert isinstance(multi_index2, MultiIndex)
        #if not same_multi_index(multi_index, multi_index2):
        #    _assign_error()

    if isinstance(expr, Function):
        linear_comb.append((expr, scalar_weight))

    elif isinstance(expr, (ComponentTensor, Product, Division, Sum)):
        # If componentTensor we need to unpack the MultiIndices
        if isinstance(expr, ComponentTensor):
            expr, multi_index = expr.ufl_operands
            #if not same_multi_index(multi_index, multi_index2):
            #    _error()

        if isinstance(expr, (Product, Division)):
            linear_comb = _check_mul_and_division(expr, linear_comb, scalar_weight, multi_index)
        elif isinstance(expr, Sum):
            linear_comb = _check_and_extract_functions(expr, linear_comb, scalar_weight, multi_index)
        else:
            _assign_error()
    else:
        _assign_error()

    return linear_comb

def _check_and_extract_functions(e, linear_comb=None, scalar_weight=1.0,
                                 multi_index=None):
    """
    Utility func for extracting Functions and scalars in linear
    combinations of Functions
    """
    from ufl.classes import ComponentTensor, Sum, Product, Division
    linear_comb = linear_comb or []

    # First check u
    if isinstance(e, Function):
        linear_comb.append((e, scalar_weight))
        return linear_comb

    # Second check a*u*b, u/a/b, a*u/b where a and b are scalars
    elif isinstance(e, (Product, Division)):
        linear_comb = _check_mul_and_division(e, linear_comb, scalar_weight, multi_index)
        return linear_comb

    # Third check a*u*b, u/a/b, a*u/b where a and b are scalars and u
    # is a Tensor
    elif isinstance(e, ComponentTensor):
        e, multi_index = e.ufl_operands
        linear_comb = _check_mul_and_division(e, linear_comb, scalar_weight, multi_index)
        return linear_comb

    # If not Product or Division we expect Sum
    elif isinstance(e, Sum):
        for op in e.ufl_operands:
            linear_comb = _check_and_extract_functions(op, linear_comb, \
                                                       scalar_weight, multi_index)

    else:
        _assign_error()

    return linear_comb

def _check_and_contract_linear_comb(expr, self, multi_index):
    """
    Utility func for checking and contracting linear combinations of
    Functions
    """
    linear_comb = _check_and_extract_functions(expr, multi_index=multi_index)
    funcs = []
    weights = []
    funcspace = None
    for func, weight in linear_comb:
        funcspace = funcspace or func.function_space()
        if func not in funcspace:
            _assign_error()
        try:
            # Check if the exact same Function is already present
            ind = funcs.index(func)
            weights[ind] += weight
        except:
            funcs.append(func)
            weights.append(weight)

    # Check that rhs does not include self
    for ind, func in enumerate(funcs):
        if func == self:
            # If so make a copy
            funcs[ind] = self.copy(deepcopy=True)
            break

    return list(zip(funcs, weights))

class Function(ufl.Coefficient, cpp.Function):
    """This class represents a function :math:`u_h` in a finite
    element function space :math:`V_h`, given by

    .. math::

        u_h = \sum_{i=1}^n U_i \phi_i,

    where :math:`\{\phi_i\}_{i=1}^n` is a basis for :math:`V_h`,
    and :math:`U` is a vector of expansion coefficients for
    :math:`u_h`.

    *Arguments*
        There is a maximum of three arguments. The first argument must be a
        Function or a :py:class:`FunctionSpace
        <dolfin.functions.functionspace.FunctionSpace>`.

        If instantiated from another Function, the (optional)
        second argument must be an integer denoting the number
        of sub functions to extract.

        In addition can a name argument be passed overruling the default name

    *Examples*
        Create a Function:

        - from a :py:class:`FunctionSpace
          <dolfin.functions.functionspace.FunctionSpace>` ``V``

          .. code-block:: python

              f = Function(V)

        - from cpp.Function ``f``

          *Warning: this constructor is intended for internal libray use only.*
          No copying is done - ``f`` is only wrapped as Function.

          .. code-block:: python

              g = Function(f)

        - from a :py:class:`FunctionSpace
          <dolfin.functions.functionspace.FunctionSpace>` ``V`` and a
          :py:class:`GenericVector <dolfin.cpp.GenericVector>` ``v``

          *Warning: this constructor is intended for internal libray use only.*

          .. code-block:: python

              g = Function(V, v)

        - from a :py:class:`FunctionSpace
          <dolfin.functions.functionspace.FunctionSpace>` and a
          filename containg a :py:class:`GenericVector
          <dolfin.cpp.GenericVector>`

          .. code-block:: python

              g = Function(V, 'MyVectorValues.xml')

    """

    def __init__(self, *args, **kwargs):
        """Initialize Function."""
        # Initial quick check for valid arguments (other checks
        # sprinkled below)
        if len(args) == 0:
            raise TypeError("expected 1 or more arguments")

        # Type switch on argument types
        if isinstance(args[0], Function):
            other = args[0]
            if len(args) == 1:
                # Copy constructor used to be here
                raise RuntimeError("Use 'Function.copy(deepcopy=True)' for copying.")
            elif len(args) == 2:
                i = args[1]
                if not isinstance(i, int):
                    raise TypeError("Invalid subfunction number %s" % (i,))
                self.__init_subfunction_constructor(other, i)
            else:
                raise TypeError("expected one or two arguments when "
                                "instantiating from another Function")
        elif isinstance(args[0], cpp.Function):
            other = args[0]
            if len(args) == 1:
                # If creating a dolfin.Function from a cpp.Function
                self.__init_from_cpp_function(other)
            else:
                raise TypeError("expected only one argument when passing cpp.Function"
                                "to dolfin.Function constructor")
        elif isinstance(args[0], FunctionSpace):
            V = args[0]
            # If initialising from a FunctionSpace
            if len(args) == 1:
                # If passing only the FunctionSpace
                self.__init_from_function_space(V)
            elif len(args) == 2:
                # If passing FunctionSpace together with cpp.Function
                # Attached passed FunctionSpace and initialize the
                # cpp.Function using the passed Function
                other = args[1]
                if isinstance(other, cpp.Function):
                    self.__init_from_function_space_and_cpp_function(V, other)
                else:
                    self.__init_from_function_space_and_function(V, other)
            else:
                raise TypeError("too many arguments")
        else:
            raise TypeError("expected a FunctionSpace or a Function as argument 1")

        # Set name as given or automatic
        name = kwargs.get("name") or "f_%d" % self.count()
        self.rename(name, "a Function")

    def __init_from_cpp_function(self, other):
        # Assign all the members (including 'this' pointer to SWIG wraper)
        # NOTE: This in fact performs assignment of C++ context
        self.__dict__ = other.__dict__

        # Initialize the ufl.FunctionSpace (Not calling cpp.Function.__init__)
        ufl.Coefficient.__init__(self, other.function_space().ufl_function_space(), count=self.id())

    def __init_subfunction_constructor(self, other, i):
        num_sub_spaces = other.function_space().num_sub_spaces()
        if num_sub_spaces == 1:
            raise RuntimeError("No subfunctions to extract")
        if not i < num_sub_spaces:
            raise RuntimeError("Can only extract subfunctions "
                               "with i = 0..%d"% num_sub_spaces)
        cpp.Function.__init__(self, other, i)
        ufl.Coefficient.__init__(self, self.function_space().ufl_function_space(), count=self.id())

    def __init_from_function_space(self, V):
        cpp.Function.__init__(self, V)
        ufl.Coefficient.__init__(self, V.ufl_function_space(), count=self.id())

    def __init_from_function_space_and_cpp_function(self, V, other):
        # Simple consistency checks on function spaces
        if other.function_space().dim() != V.dim():
            raise ValueError("non matching dimensions on passed FunctionSpaces")
        cpp.Function.__init__(self, other)
        ufl.Coefficient.__init__(self, V.ufl_function_space(), count=self.id())

    def __init_from_function_space_and_function(self, V, other):
        cpp.Function.__init__(self, V, other)
        ufl.Coefficient.__init__(self, V.ufl_function_space(), count=self.id())

    def sub(self, i, deepcopy = False):
        """
        Return a sub function.

        The sub functions are numbered from i = 0..N-1, where N is the
        total number of sub spaces.

        *Arguments*
            i : int
                The number of the sub function

        """
        if not isinstance(i, int):
            raise TypeError("expects an 'int' as first argument")
        num_sub_spaces = self.function_space().num_sub_spaces()
        if num_sub_spaces == 1:
            raise RuntimeError("No subfunctions to extract")
        if not i < num_sub_spaces:
            raise RuntimeError("Can only extract subfunctions with i = 0..%d" \
                               % num_sub_spaces)

        # Create and instantiate the Function
        if deepcopy:
            return Function(self.function_space().sub(i), \
                            cpp.Function.sub(self, i), \
                            name='%s-%d' % (str(self), i))
        else:
            return Function(self, i, name='%s-%d' % (str(self), i))

    def assign(self, rhs):
        """
        Assign either a Function or linear combination of Functions.

        *Arguments*
            rhs (_Function_)
                A Function or a linear combination of Functions. If a linear
                combination is passed all Functions need to be in the same
                FunctionSpaces.
        """
        from ufl.classes import ComponentTensor, Sum, Product, Division
        if isinstance(rhs, (cpp.Function, cpp.Expression, cpp.FunctionAXPY)):
            # Avoid self assignment
            if self == rhs:
                return

            self._assign(rhs)
        elif isinstance(rhs, (Sum, Product, Division, ComponentTensor)):
            if isinstance(rhs, ComponentTensor):
                rhs, multi_index = rhs.ufl_operands
            else:
                multi_index = None
            linear_comb = _check_and_contract_linear_comb(rhs, self, \
                                                          multi_index)
            assert(linear_comb)

            # If the assigned Function lives in a different FunctionSpace
            # we cannot operate on this function directly
            same_func_space = linear_comb[0][0] in self.function_space()
            func, weight = linear_comb.pop()

            # Assign values from first func
            if not same_func_space:
                self._assign(func)
                vector = self.vector()
            else:
                vector = self.vector()
                vector[:] = func.vector()

            # If first weight is not 1 scale
            if weight != 1.0:
                vector *= weight

            # AXPY the other functions
            for func, weight in linear_comb:
                if weight == 0.0:
                    continue
                vector.axpy(weight, func.vector())

        else:
            cpp.dolfin_error("function.py",
                             "function assignment",
                             "Expects a Function or linear combinations of "\
                             "Functions in the same FunctionSpaces")

    def split(self, deepcopy=False):
        """
        Extract any sub functions.

        A sub function can be extracted from a discrete function that
        is in a mixed, vector, or tensor FunctionSpace. The sub
        function resides in the subspace of the mixed space.

        *Arguments*
            deepcopy
                Copy sub function vector instead of sharing

        """

        num_sub_spaces = self.function_space().num_sub_spaces()
        if num_sub_spaces == 1:
            raise RuntimeError("No subfunctions to extract")
        return tuple(self.sub(i, deepcopy) for i in range(num_sub_spaces))

    def __str__(self):
        """Return a pretty print representation of it self.
        """
        return self.name()

    def __repr__(self):
        """Return a str repr of it self.

        Must use ufl.__repr__ for this"""
        return ufl.Coefficient.__repr__(self)

    def str(self, verbose=False):
        """Return an informative str representation of itself"""
        # FIXME: We might change this using rank and dimension instead
        return "<Function in %s>" % str(self.function_space())

    def ufl_evaluate(self, x, component, derivatives):
        """Function used by ufl to evaluate the Function"""
        import numpy
        import ufl
        assert derivatives == () # TODO: Handle derivatives

        if component:
            shape = self.ufl_shape
            assert len(shape) == len(component)
            value_size = product(shape)
            index = flatten_multiindex(component, shape_to_strides(shape))
            values = numpy.zeros(value_size)
            self(*x, values=values)
            return values[index]
        else:
            # Scalar evaluation
            return self(*x)

    def __float__(self):
        if self.ufl_shape != ():
            raise RuntimeError("Cannot convert nonscalar function to float.")
        elm = self.ufl_element()
        if elm.family() != "Real":
            raise RuntimeError("Cannot convert spatially varying function to float.")
        # FIXME: This could be much simpler be exploiting that the
        # vector is ghosted
        # Gather value directly from vector in a parallel safe way
        vec = self.vector()
        indices = numpy.zeros(1, dtype=cpp.la_index_dtype())
        values = vec.gather(indices)
        return float(values[0])

    def __call__(self, *args, **kwargs):
        """
        Evaluates the Function.

        *Examples*
            1) Using an iterable as x:

              .. code-block:: python

                  fs = Expression("sin(x[0])*cos(x[1])*sin(x[3])")
                  x0 = (1.,0.5,0.5)
                  x1 = [1.,0.5,0.5]
                  x2 = numpy.array([1.,0.5,0.5])
                  v0 = fs(x0)
                  v1 = fs(x1)
                  v2 = fs(x2)

            2) Using multiple scalar args for x, interpreted as a
            point coordinate

              .. code-block:: python

                  v0 = f(1.,0.5,0.5)

            3) Using a Point

              .. code-block:: python

                  p0 = Point(1.,0.5,0.5)
                  v0 = f(p0)

            3) Passing return array

              .. code-block:: python

                  fv = Expression(("sin(x[0])*cos(x[1])*sin(x[3])",
                               "2.0","0.0"))
                  x0 = numpy.array([1.,0.5,0.5])
                  v0 = numpy.zeros(3)
                  fv(x0, values = v0)

              .. note::

                  A longer values array may be passed. In this way one can fast
                  fill up an array with different evaluations.

              .. code-block:: python

                  values = numpy.zeros(9)
                  for i in xrange(0,10,3):
                      fv(x[i:i+3], values = values[i:i+3])

        """

        if len(args)==0:
            raise TypeError("expected at least 1 argument")

        # Test for ufl restriction
        if len(args) == 1 and isinstance(args[0], string_types):
            if args[0] in ('+', '-'):
                return ufl.Coefficient.__call__(self, *args)

        # Test for ufl mapping
        if len(args) == 2 and isinstance(args[1], dict) and self in args[1]:
            return ufl.Coefficient.__call__(self, *args)

        # Some help variables
        value_size = product(self.ufl_element().value_shape())

        # If values (return argument) is passed, check the type and length
        values = kwargs.get("values", None)
        if values is not None:
            if not isinstance(values, numpy.ndarray):
                raise TypeError("expected a NumPy array for 'values'")
            if len(values) != value_size or \
                   not numpy.issubdtype(values.dtype, 'd'):
                raise TypeError("expected a double NumPy array of length"\
                      " %d for return values."%value_size)
            values_provided = True
        else:
            values_provided = False
            values = numpy.zeros(value_size, dtype='d')

        # Get the geometric dimension we live in
        dim = self.ufl_domain().geometric_dimension()

        # Assume all args are x argument
        x = args

        # If only one x argument has been provided, unpack it if it's
        # an iterable
        if len(x) == 1:
            if isinstance(x[0], cpp.Point):
                x = [x[0][i] for i in range(dim)]
            elif hasattr(x[0], '__iter__'):
                x = x[0]

        # Convert it to an 1D numpy array
        try:
            x = numpy.fromiter(x, 'd')
        except (TypeError, ValueError, AssertionError) as e:
            raise TypeError("expected scalar arguments for the coordinates")

        if len(x) == 0:
            raise TypeError("coordinate argument too short")

        if len(x) != dim:
            raise TypeError("expected the geometry argument to be of "\
                  "length %d"%dim)

        # The actual evaluation
        self.eval(values, x)

        # If scalar return statement, return scalar value.
        if value_size == 1 and not values_provided:
            return values[0]

        return values

#--- Subclassing of ufl.{Basis, Trial, Test}Function ---

# TODO: Update this message to clarify dolfin.FunctionSpace vs ufl.FunctionSpace
_ufl_dolfin_difference_message = """\
When constructing an Argument, TestFunction or TrialFunction,
you must to provide a FunctionSpace and not a FiniteElement.
The FiniteElement class provided by ufl only represents an
abstract finite element space and is only used in standalone
.ufl files, while the FunctionSpace provides a full discrete
function space over a given mesh and should be used in dolfin
programs in Python.
"""

class Argument(ufl.Argument):
    """UFL value: Representation of an argument to a form.

    This is the overloaded PyDOLFIN variant.
    """
    def __init__(self, V, number, part=None):

        # Check argument
        if not isinstance(V, (FunctionSpace, cpp.MultiMeshFunctionSpace)):
            if isinstance(V, (ufl.FiniteElementBase, ufl.FunctionSpace)):
                raise TypeError(_ufl_dolfin_difference_message)
            else:
                raise TypeError("Illegal argument for creation of Argument, not a FunctionSpace: " + str(V))
            raise TypeError("Illegal argument for creation of Argument, not a FunctionSpace: " + str(V))

        # Handle MultiMesh
        if isinstance(V, cpp.MultiMeshFunctionSpace):
            self._V_multi = V
            V = V._parts[0]

        # Initialize UFL Argument
        ufl.Argument.__init__(self, V.ufl_function_space(), number, part)

        self._V = V

    def function_space(self):
        "Return the FunctionSpace"
        return self._V

    def __eq__(self, other):
        """Extending UFL __eq__ here to distinguish test and trial
        functions in different function spaces with same ufl element."""
        return (isinstance(other, Argument) and
                self.number() == other.number() and
                self.part() == other.part() and
                self._V == other._V)

    def __hash__(self):
        return ufl.Argument.__hash__(self)

def TestFunction(V, part=None):
    """UFL value: Create a test function argument to a form.

    This is the overloaded PyDOLFIN variant.
    """
    return Argument(V, 0, part)

def TrialFunction(V, part=None):
    """UFL value: Create a trial function argument to a form.

    This is the overloaded PyDOLFIN variant.
    """
    return Argument(V, 1, part)

#--- TestFunctions and TrialFunctions ---

def Arguments(V, number):
    """UFL value: Create an Argument in a mixed space, and return a
    tuple with the function components corresponding to the subelements.

    This is the overloaded PyDOLFIN variant.
    """
    return ufl.split(Argument(V, number))

def TestFunctions(V):
    """UFL value: Create a TestFunction in a mixed space, and return a
    tuple with the function components corresponding to the subelements.

    This is the overloaded PyDOLFIN variant.
    """
    return ufl.split(TestFunction(V))

def TrialFunctions(V):
    """UFL value: Create a TrialFunction in a mixed space, and return a
    tuple with the function components corresponding to the subelements.

    This is the overloaded PyDOLFIN variant.
    """
    return ufl.split(TrialFunction(V))



# -*- coding: utf-8 -*-
"""The function module of dolfin"""
from dolfin.fem import form
from dolfin.fem import assembling
from dolfin.fem import norms
from dolfin.fem import projection
from dolfin.fem import interpolation
from dolfin.fem import bcs
from dolfin.fem import formmanipulations

from .form import *
from .assembling import *
from .assembling_local import *
from .solving import *
from .adaptivesolving import *
from .interpolation import *
from .norms import *
from .projection import *
from .bcs import *
from .formmanipulations import *

# NOTE: The automatic documentation system in DOLFIN requires to _not_ define
# NOTE: classes or functions within this file. Use separate modules for that
# NOTE: purpose.

__all__ = form.__all__ + \
          assembling.__all__ + \
          assembling_local.__all__ + \
          solving.__all__ + \
          adaptivesolving.__all__ + \
          norms.__all__ + \
          projection.__all__ + \
          interpolation.__all__ + \
          bcs.__all__ + \
          formmanipulations.__all__



# -*- coding: utf-8 -*-
"This module provides uniform and local mesh refinement."

# Copyright (C) 2009 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2010-02-26
# Last changed: 2010-02-26

__all__ = ["refine"]

# Import C++ interface
import dolfin.cpp as cpp

def refine(mesh, cell_markers=None, redistribute=True):
    """
    Refine given mesh and return the refined mesh.

    *Arguments*
        mesh
            the :py:class:`Mesh <dolfin.cpp.Mesh>` to be refined.
        cell_markers
            an optional argument (a boolean :py:class:`MeshFunction
            <dolfin.cpp.MeshFunctionBool>` over cells) may be given
            to specify which cells should be refined. If no cell
            markers are given, then the mesh is refined uniformly.
        redistribute
            an optional argument (boolean) may be given to set whether
            to redistribute the mesh across processes following
            refinement. The is relevant only when the mesh is
            distributed across processes. In serial the argument has
            no impact. Default it True.

    *Examples of usage*

        .. code-block:: python

            mesh = refine(mesh)

        To only refine cells with too large error, define a boolean
        MeshFunction over the mesh and mark the cells to be refined
        as True.

        .. code-block:: python

            cell_markers = CellFunction("bool", mesh)
            cell_markers.set_all(False)
            for cell in cells(mesh):
                # set cell_markers[cell] = True if the cell's error
                # indicator is greater than some criterion.
            mesh = refine(mesh, cell_markers)

    """

    # This function is necessary to avoid copying in the wrapped
    # versions of the return-by-value refine functions in C++.

    # Create empty mesh
    refined_mesh = cpp.Mesh(mesh.mpi_comm())

    # Call C++ refinement
    if cell_markers is None:
        cpp.refine(refined_mesh, mesh, redistribute)
    else:
        cpp.refine(refined_mesh, mesh, cell_markers, redistribute)

    return refined_mesh



# -*- coding: utf-8 -*-
#!/usr/bin/env python

# Copyright (C) 2009 Martin Sandve Alnæs
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

# Modified Johan Hake 2011

from __future__ import print_function
import os

# Template for script sent to qsub
template = """#!/bin/bash
%(pbs)s

# enter the job-directory
cd %(workdir)s

# add /usr/local/bin in path
PATH=$PATH:/usr/local/bin

# setup environment
%(paths)s
%(setup)s

# execute job
%(job)s
"""

def job_script(job, jobname, nodes, ppn, walltime, mem, vmem, workdir, \
               email, paths, setup, keep_environment, queue, parallel_environment):
    """
    Generate a torque specific job script
    """
    if queue or parallel_environment:
        print("Warning: 'queue' and 'parallel_environment' is not available "\
              "for the 'torque' backend")

    # Set working dir
    if workdir == os.path.abspath(os.getcwd()):
        workdir = "$PBS_O_WORKDIR"
    
    pbs = []
    pbs.append("-j oe")
    pbs_resources = "-lnodes=%d:ppn=%d,walltime=%d:00:00" % (nodes, ppn, walltime)
    if mem:
        pbs_resources += ",mem=" + mem
    if vmem:
        pbs_resources += ",vmem=" + vmem
    pbs.append(pbs_resources)
    if email:
        pbs.append("-m abe")
        pbs.append("-M %s" % email)
    pbs.append("-N %s" % jobname)
    if keep_environment:
        pbs.append("-V")
    pbs.append("")
    pbs = "\n".join("#PBS %s" % a for a in pbs)
    
    args = dict(pbs=pbs,
                workdir=workdir,
                paths=paths,
                setup=setup,
                job=job)
    
    return template % args



"""
This demo program illustrates how to solve Poisson's equation

    - div grad u(x, y) = f(x, y)

on the unit square with pure Neumann boundary conditions:

    du/dn(x, y) = -sin(5*x)

and source f given by

    f(x, y) = 10*exp(-((x - 0.5)^2 + (y - 0.5)^2) / 0.02)

Since only Neumann conditions are applied, u is only determined up to
a constant by the above equations. An addition constraint is thus
required, for instance

  \int u = 0

This is accomplished in this demo by using a Krylov iterative solver
that removes the component in the null space from the solution vector.
"""

# Copyright (C) 2012-2016 Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Begin demo

from dolfin import *
import matplotlib.pyplot as plt

# Test for PETSc
if not has_linear_algebra_backend("PETSc"):
    info("DOLFIN has not been configured with PETSc. Exiting.")
    exit()

parameters["linear_algebra_backend"] = "PETSc"

# Create mesh and define function space
mesh = UnitSquareMesh(128, 128)
V = FunctionSpace(mesh, "CG", 1)

# Define variational problem
u = TrialFunction(V)
v = TestFunction(V)
f = Expression("10*exp(-(pow(x[0] - 0.5, 2) + pow(x[1] - 0.5, 2)) / 0.02)",
               degree=2)
g = Expression("-sin(5*x[0])", degree=2)
a = inner(grad(u), grad(v))*dx
L = f*v*dx + g*v*ds

# Assemble system
A = assemble(a)
b = assemble(L)

# Create vector that spans the null space, and normalize
null_space_vector = b.copy()
null_space_vector[:] = sqrt(1.0/null_space_vector.size())

# Create null space basis object and attach to PETSc matrix
null_space = VectorSpaceBasis([null_space_vector])
as_backend_type(A).set_nullspace(null_space)

# Orthogonalize RHS vector b with respect to the null space (this
# ensures that a solution exists)
null_space.orthogonalize(b);

# Set PETSc solve type (conjugate gradient) and preconditioner
# (algebraic multigrid)
PETScOptions.set("ksp_type", "cg")
PETScOptions.set("pc_type", "gamg")

# Since we have a singular problem, use SVD solver on the multigrid
# 'coarse grid'
PETScOptions.set("mg_coarse_ksp_type", "preonly")
PETScOptions.set("mg_coarse_pc_type", "svd")

# Set the solver tolerance
PETScOptions.set("ksp_rtol", 1.0e-8)

# Print PETSc solver configuration
PETScOptions.set("ksp_view")
PETScOptions.set("ksp_monitor")

# Create Krylov solver and set operator
solver = PETScKrylovSolver()
solver.set_operator(A)

# Set PETSc options on the solver
solver.set_from_options()

# Create solution Function
u = Function(V)

# Solve
solver.solve(u.vector(), b)

# Plot solution
plot(u)
plt.show()



# -*- coding: utf-8 -*-
#!/usr/bin/env python

# Copyright (C) 2013 Johannes Ring
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

template = """#!/bin/bash
%(sbatch)s

%(paths)s

%(setup)s

%(job)s
"""

def job_script(job, jobname, nodes, ppn, walltime, mem, vmem, workdir, \
               email, paths, setup, keep_environment, queue, parallel_environment):
    """
    Generate a slurm specific job script.
    """

    sbatch = "#SBATCH --job-name=" + jobname
    sbatch += " --time=%d:00:00" % walltime
    sbatch += " --ntasks=" + str(nodes)
    sbatch += " --cpus-per-task=" + str(ppn)

    if mem:
        sbatch += " --mem-per-cpu=" + mem
    if email:
        sbatch += " --mail-user=" + email
    if keep_environment:
        sbatch += " --get-user-env"

    # Issue warnings on not supported arguments
    if vmem:
        print("Warning: 'vmem' is not supported for the 'slurm' backend")
    if queue:
        print("Warning: 'queue' is not supported for the 'slurm' backend")
    if parallel_environment:
        print("Warning: 'parallel_environment' is not supported for the 'slurm' backend")

    args = dict(sbatch=sbatch,
                workdir=workdir,
                paths=paths,
                setup=setup,
                job=job)

    return template % args



# -*- coding: utf-8 -*-
"""Shared fixtures for unit tests involving dolfin."""

# Copyright (C) 2014-2014 Martin Sandve Alnæs and Aslak Wigdahl Bergersen
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function
import pytest
import os
import shutil
import tempfile
import gc
import platform
from instant import get_status_output
from dolfin import  *

from .fixtures import filedir, gc_barrier
from .skips import skip_in_parallel

@skip_in_parallel
@pytest.mark.cpp
def cpp_tester(request):
    gc_barrier()
    filedir = os.path.dirname(os.path.abspath(request.module.__file__))

    prefixes = ["", "mpirun -np 3 "]
    curdir = os.getcwd()
    os.chdir(filedir)
    try:
        # Set non-interactive, configure and build
        os.putenv('DOLFIN_NOPLOT', '1')

        #status, out_cmake = get_status_output('cmake .')
        #assert status == 0

        #status, out_make = get_status_output('make')
        #assert status == 0

        # Find all built test binaries
        list_of_tests = []
        for file in sorted(os.listdir('.')):
            if file.startswith("test_") and not '.' in file:
                list_of_tests.append(file)
        assert list_of_tests, "C++ tests have not been built!"

        # Run test binaries
        for prefix in prefixes: # With or without mpirun
            for test in list_of_tests:
                cmd = prefix + os.path.join('.', test)
                print("Running cmd: ", cmd)
                status, output = get_status_output(cmd)
                if not (status == 0 and 'OK' in output):
                    print(output)
                assert status == 0 and 'OK' in output

    finally:
        os.chdir(curdir)

    gc_barrier()



# -*- coding: utf-8 -*-
"""The mesh module of dolfin"""
from dolfin.mesh import meshes
from dolfin.mesh import ale
from dolfin.mesh import refinement

from .meshes import *
from .ale import *
from .refinement import *

# NOTE: The automatic documentation system in DOLFIN requires to _not_
# NOTE: define classes or functions within this file. Use separate
# NOTE: modules for that purpose.

__all__ = meshes.__all__ + ale.__all__ + refinement.__all__



# -*- coding: utf-8 -*-
"""Expose names from the compiled cpp modules to the dolfin.cpp
namespace"""

# Copyright (C) 2012 Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Garth N. Wells, 2013
#
# First added:  2012-01-29
# Last changed: 2013-11-01

__all__ = []

#--- Imports the SWIG-generated Python code (C++ interface) ---

# NOTE: Order of these imports need to be preserved
_compiled_modules = ["common", "la", "function", "io", "mesh", "fem"]

# Prohibit includes with these names
_not_includes = _compiled_modules[:]
_not_includes.extend(["ufc", "new_instancemethod", "SHARED_PTR_DISOWN", \
                      "weakref", "weakref_proxy", "cvar"])

# Import all compiled modules
for module_name in _compiled_modules:

    # Import the module
    exec("from . import %s" % module_name)
    module = globals()[module_name]

    # Add the module to the global namespace
    __all__.append(module_name)

    # Collect all classes to outrule static class methods
    _all_classes = []

    # Iterate over the modules attributes and add names to global namespace
    for name in sorted(module.__dict__.keys()):

        # Skip some includes
        if name in _not_includes:
            continue

        # Do not add an attrbute name which contains "swigregister" or
        # starts with "_"
        if "swigregister" in name or name[0] == "_":
            continue

        # Get attr
        attr = getattr(module, name)

        # Check for class
        if isinstance(attr, type):
            _all_classes.append(name)
        else:
            # If not a class, check if first part of function name is
            # the same as a class name
            if name.split("_")[0] in _all_classes:
                continue

        # Add the attribute to the global namespace
        globals()[name] = attr
        __all__.append(name)

# Add versions
# -----------
globals()["__swigversion__"] = getattr(module, "__swigversion__")
globals()["__pythonversion__"] = getattr(module, "__pythonversion__")
globals()["__version__"] = common.dolfin_version()
globals()["__ufcsignature__"] = common.ufc_signature()
globals()["parameters"] = common.get_global_parameters()



# -*- coding: utf-8 -*-
"This module provides a simple Python wrapper for the SWIG-generated Mesh class"

# Copyright (C) 2012-2014 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Marie E. Rognes (meg@simula.no)
# Modified by Martin Sandve Alnæs (martinal@simula.no) 2014

__all__ = []

# Import ufl (used by Mesh extensions in dolfin/swig/mesh/post.i)
import ufl

# Import SWIG-generated extension module (DOLFIN C++)
import dolfin.cpp as cpp

# Replacement constructor for Mesh class. We cannot create a Python
# subclass of cpp.Mesh since then isinstance(UnitSquare, Mesh) would
# not be true.

def __init__(self, *args, **kwargs):
    """Create a mesh from a filename or a geometry.

    A mesh may be created from a given filename, which should contain
    mesh data stored in DOLFIN XML format:

    .. code-block:: python

        mesh = Mesh("mesh.xml")

    An empty mesh may be created as follows:

    .. code-block:: python

        mesh = Mesh()

    A copy of a mesh may be created as follows:

    .. code-block:: python

        mesh_copy = Mesh(mesh)
    """

    # Reuse C++ doc-string too
    __doc__ = cpp.Mesh.__doc__

    # Initialize base class
    cpp.Mesh.__cppinit__(self, *args, **kwargs)

# Change mesh constructor
cpp.Mesh.__cppinit__ = cpp.Mesh.__init__
cpp.Mesh.__init__ = __init__

def mesh2html(mesh, display_width=800.0):
    return cpp.X3DOM.html(mesh)

# Attach ipython notebook svg rendering function for 1D and 2D meshes
from dolfin.mesh.svgtools import mesh2svg
cpp.Mesh._repr_svg_ = mesh2svg
cpp.Mesh._repr_html_ = mesh2html



# -*- coding: utf-8 -*-
"""The importhandler module of dolfin"""

# NOTE: The automatic documentation system in DOLFIN requires to _not_ define
# NOTE: classes or functions within this file. Use separate modules for that
# NOTE: purpose.

# System imports
from __future__ import print_function
import sys

# Import NumPy so it will not be screwed by imports after RTLD_GLOBAL is set
# Work around for lp:1085986
import numpy

# Work around lp:956939 and lp:961946
import io

# Store dl open flags to restore them after import
stored_dlopen_flags = sys.getdlopenflags()

# A try to get rid of problems with dynamic_cast of types defined in different
# SWIG modules. The problem rises from the dynamic loader not making all types
# available for all shared modules. The key is the RTLD_GLOBAL. RTLD_NOW is set
# by default. This seems to be similar to the dynamicloadmpi problem above.
# See: for references: http://gcc.gnu.org/faq.html#dso
if "linux" in sys.platform:
    # FIXME: What with other platforms?
    try:
        from ctypes import RTLD_NOW, RTLD_GLOBAL
    except ImportError:
        RTLD_NOW = 2
        RTLD_GLOBAL = 256
    sys.setdlopenflags(RTLD_NOW | RTLD_GLOBAL)

# Check for runtime dependencies
runtime_dependencies = ["ufl", "ffc", "ply.lex"]

for dep in runtime_dependencies:
    try:
        __import__(dep)
    except ImportError as e:
        import sys
        message = ["DOLFIN runtime dependency is not met.",
                   "Install the following python module: '%s' "  % dep,
                   "and make sure its location is listed in PYTHONPATH."]
        wrap = "-"*len(max(message, key=len))
        print()
        print(wrap)
        for s in message:
            print(s)
        print(wrap)
        print()
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright (C) 2011 Anders Logg, Kristian Oelgaard, and Marie E. Rognes
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2011-05-23
# Last changed: 2011-05-27

library_index_cpp_template = """\
.. _programmers_reference_index:

#############################################
C++ Programmer's reference for DOLFIN-%s
#############################################

.. toctree::
   :glob:
   :maxdepth: 2

   */index
"""

module_index_cpp_template = """\
.. _programmers_reference_%s_index:

**********************************
DOLFIN %s module
**********************************

.. toctree::
   :glob:
   :maxdepth: 1

   *
"""

header_cpp_template = """\

.. Documentation for the header file dolfin/%(module)s/%(header)s

.. _programmers_reference_cpp_%(module)s_%(prefix)s:

%(title)s
%(formatting)s

.. note::
%(note)s

%(text)s
"""

note_template = """
The documentation on this page was automatically extracted from the
DOLFIN C++ code and may need to be edited or expanded.
"""

automodule_template = """

%(title)s
%(formatting)s

.. automodule:: %(module)s
   :members:
   :undoc-members:
   :show-inheritance:
"""

def generate_library_index(version):
    return library_index_cpp_template % version

def generate_module_index(module):
    return module_index_cpp_template % (module, module)

def generate_automodule(title, formatting, module_name):
    options = {"module": module_name, "title": title,
               "formatting": len(title)*formatting}
    return automodule_template % options





# -*- coding: utf-8 -*-
"""Main module for DOLFIN"""

# Special module to handle import issues
import dolfin.importhandler

# Import UFL form language
from ufl import *

# Add log level present in C++ DOLFIN but not ufl.log
PROGRESS = 16

# Remove some unnecessary imports from UFL
del set_handler, set_level

# Import names from the compiled cpp modules
from . import cpp
from dolfin.cpp import *
from dolfin.cpp import __version__, __swigversion__, __pythonversion__

# Reset dl open flags
import sys
sys.setdlopenflags(dolfin.importhandler.stored_dlopen_flags)
del sys

# Repair namespace
from ufl import FiniteElement

# Import UFL color printing
from ufl.log import info_red, info_green, info_blue

# List of supported elements from FFC
try:
    from ffc import supported_elements, supported_elements_for_plotting
except:
    supported_elements = []
    supported_elements_for_plotting = []

# Imports from pure Python modules
from dolfin.common import *
from dolfin.functions import *
from dolfin.compilemodules import *
from dolfin.fem import *
from dolfin.mesh import *
from dolfin.multistage import *

# Handling of deprecated interfaces
from .deprecation import *




# -*- coding: utf-8 -*-
# Copyright (C) 2010 Anders Logg and Kristian Oelgaard
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Marie E. Rognes
#
# First added:  2010
# Last changed: 2011-05-23

from __future__ import print_function
import os

# Import the documentation extractor tools from dolfin_utils
from dolfin_utils.documentation import extract_doc_representation
from dolfin_utils.documentation import indent, add_links

from . import indextemplates as templates

__all__ = ["generate_cpp_api_documentation"]

def write_class_doc(classname, parents, comment, function_documentation,
                    classnames):

    output = []
    base_indent = 0
    if classname is not None:
        base_indent = 4
        output += [".. cpp:class:: %s\n" % classname]

        if parents is not None:
            output += [indent("*Parent class(es)*\n", 4)]
            for parent in parents:
                output += [indent("* :cpp:class:`%s`\n" % parent, 8)]

        if comment is not None:
            comment = add_links(comment, classnames, ":cpp:class:") #NB
            output += [indent(comment, 4)] + ["\n"]

    else:
        output += ["\n"]

    for (signature, comment) in function_documentation:

        # Handle indentation of signature # Hm?
        if "\n" in signature:
            lines = signature.split("\n")
            signature = ("\n".join([lines[0]]
                                   + [(len(".. cpp.function::") + 1)*" " + l
                                      for l in lines[1:]]))
        # Add crosslinks in comment
        comment = add_links(comment, classnames, ":cpp:class:")

        output += [indent(".. cpp:function:: %s\n" % signature, base_indent)]
        output += [indent(comment, base_indent + 4), "\n"]

    text = "\n".join(output)
    return text

def generate_header_rst_file(module, header, documentation, classnames):

    output = []
    for (classname, parents, comment, function_documentation) in documentation:
        output += [write_class_doc(classname, parents, comment,
                                   function_documentation, classnames)]

    text = "\n".join(output)
    note = indent(templates.note_template, 4)

    dictionary = {"module": module, "header": header,
                  "prefix": header.split(".")[0].lower(),
                  "title": header, "formatting": len(header)*"=",
                  "note": note, "text": text}
    template = templates.header_cpp_template % dictionary

    return template

def open_write_close(filename, contents):
    f = open(filename, "w")
    f.write(contents)
    f.close()

def generate_cpp_module(module, documentation, classnames, output_dir):

    print(indent("Generating documentation for %s ... " % module, 0))

    # Generate index file for this module
    module_index = templates.generate_module_index(module)
    index_filename = os.path.join(output_dir, "index.rst")
    open_write_close(index_filename, module_index)

    # Generate .rst for each header file in this module
    for (header, doc) in documentation:

        #print indent("Generating documentation for %s" % header, 2)
        rst_filename = os.path.join(output_dir, header.split(".")[0] + ".rst")
        header_rst = generate_header_rst_file(module, header, doc,
                                              classnames)
        open_write_close(rst_filename, header_rst)

    return

def generate_cpp_api_documentation(input_dir, output_dir, version=None):
    """
    Generate .rst files for C++ DOLFIN Programmer's Reference

    Arguments:
      * version
        version should be the version number, example 0.9.11 or dev
    """

    print("Generating .rst for C++ API documentation of DOLFIN-%s" % version)
    print("This can take some time ...")
    print("Reading from %s" % input_dir)
    try:
        os.makedirs(output_dir)
        print("Creating output directory: %s" % output_dir)

    except:
        print("Output directory (%s) already existing" % output_dir)

    # Get representation
    documentation, classnames = extract_doc_representation(input_dir)

    # Generate index file for library
    library_index = templates.generate_library_index(version)
    library_index_filename = os.path.join(output_dir, "index.rst")
    open_write_close(library_index_filename, library_index)

    # Generate documentation for each module
    for module in documentation:

        # Create directory for this module
        module_dir = os.path.join(output_dir, module)
        try:
            os.makedirs(module_dir)
        except:
            pass

        # Create documentation for this module
        generate_cpp_module(module, documentation[module], classnames,
                            module_dir)




"""This demo program solves an elastodynamics problem."""

# Copyright (C) 2010 Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Anders Logg 2008-2011
#
# First added:  2010-04-30
# Last changed: 2012-11-12

from __future__ import print_function
from dolfin import *
import matplotlib.pyplot as plt


# Form compiler options
parameters["form_compiler"]["cpp_optimize"] = True
parameters["form_compiler"]["optimize"] = True

def update(u, u0, v0, a0, beta, gamma, dt):
    """Update fields at the end of each time step."""

    # Get vectors (references)
    u_vec, u0_vec  = u.vector(), u0.vector()
    v0_vec, a0_vec = v0.vector(), a0.vector()

    # Update acceleration and velocity

    # a = 1/(2*beta)*((u - u0 - v0*dt)/(0.5*dt*dt) - (1-2*beta)*a0)
    a_vec = (1.0/(2.0*beta))*( (u_vec - u0_vec - v0_vec*dt)/(0.5*dt*dt) - (1.0-2.0*beta)*a0_vec )

    # v = dt * ((1-gamma)*a0 + gamma*a) + v0
    v_vec = dt*((1.0-gamma)*a0_vec + gamma*a_vec) + v0_vec

    # Update (u0 <- u0)
    v0.vector()[:], a0.vector()[:] = v_vec, a_vec
    u0.vector()[:] = u.vector()

# External load
class Traction(UserExpression):

    def __init__(self, dt, t, old, **kwargs):
        self.t   = t
        self.dt  = dt
        self.old = old
        if has_pybind11():
            super().__init__(**kwargs)

    def eval(self, values, x):

        # 'Shift' time for n-1 values
        t_tmp = self.t
        if self.old and t > 0.0:
            t_tmp -= self.dt

        cutoff_t = 10.0*1.0/32.0;
        weight = t_tmp/cutoff_t if t_tmp < cutoff_t else 1.0

        values[0] = 1.0*weight
        values[1] = 0.0

    def value_shape(self):
        return (2,)

# Sub domain for clamp at left end
def left(x, on_boundary):
    return x[0] < 0.001 and on_boundary

# Sub domain for rotation at right end
def right(x, on_boundary):
    return x[0] > 0.99 and on_boundary

# Load mesh and define function space
mesh = Mesh("../dolfin_fine.xml.gz")

# Define function space
V = VectorFunctionSpace(mesh, "CG", 1)

# Test and trial functions
u = TrialFunction(V)
r = TestFunction(V)

E  = 1.0
nu = 0.0
mu    = E / (2.0*(1.0 + nu))
lmbda = E*nu / ((1.0 + nu)*(1.0 - 2.0*nu))

# Mass density andviscous damping coefficient
rho = 1.0
eta = 0.25

# Time stepping parameters
alpha_m = 0.2
alpha_f = 0.4
beta    = 0.36
gamma   = 0.7
dt      = 1.0/32.0
t       = 0.0
T       = 10*dt

# Some useful factors
factor_m1  = rho*(1.0-alpha_m)/(beta*dt*dt)
factor_m2  = rho*(1.0-alpha_m)/(beta*dt)
factor_m3  = rho*(1.0-alpha_m-2.0*beta)/(2.0*beta)

factor_d1  = eta*(1.0-alpha_f)*gamma/(beta*dt)
factor_d2  = eta*((1.0-alpha_f)*gamma-beta)/beta
factor_d3  = eta*(gamma-2.0*beta)*(1.0-alpha_f)*dt/(2.0*beta)

# Fields from previous time step (displacement, velocity, acceleration)
u0 = Function(V)
v0 = Function(V)
a0 = Function(V)

# External forces (body and applied tractions
f  = Constant((0.0, 0.0))
p  = Traction(dt, t, False, degree=1)
p0 = Traction(dt, t, True, degree=1)

# Create mesh function over the cell facets
boundary_subdomains = MeshFunction("size_t", mesh, mesh.topology().dim() - 1)
boundary_subdomains.set_all(0)
force_boundary = AutoSubDomain(right)
force_boundary.mark(boundary_subdomains, 3)

# Define measure for boundary condition integral
dss = ds(subdomain_data=boundary_subdomains)

# Stress tensor
def sigma(r):
    return 2.0*mu*sym(grad(r)) + lmbda*tr(sym(grad(r)))*Identity(len(r))

# Forms
a = factor_m1*inner(u, r)*dx + factor_d1*inner(u, r)*dx \
   +(1.0-alpha_f)*inner(sigma(u), grad(r))*dx

L =  factor_m1*inner(r, u0)*dx + factor_m2*inner(r, v0)*dx \
   + factor_m3*inner(r, a0)*dx \
   + factor_d1*inner(r, u0)*dx + factor_d2*inner(r, v0)*dx \
   + factor_d3*inner(r, a0)*dx \
   - alpha_f*inner(grad(r), sigma(u0))*dx \
   + inner(r, f)*dx + (1.0-alpha_f)*inner(r, p)*dss(3) + alpha_f*inner(r, p0)*dss(3)

# Set up boundary condition at left end
zero = Constant((0.0, 0.0))
bc = DirichletBC(V, zero, left)

# FIXME: This demo needs some improved commenting

# Time-stepping
u = Function(V)
vtk_file = File("elasticity.pvd")
while t <= T:

    t += dt
    print("Time: ", t)

    p.t = t
    p0.t = t

    solve(a == L, u, bc)
    update(u, u0, v0, a0, beta, gamma, dt)

    # Save solution to VTK format
    vtk_file << u

# Plot solution
plot(u, mode="displacement")
plt.show()
